"""


Author: 
    Inspyre Softworks

Project:
    inSPy-Logger

File: 
    inspy_logger/system/mac_os.py
 

Description:
    

"""
from inspy_logger.system.linux import get_user_name


__all__ = [
    'get_user_name'
]
